@@ -193,6 +193,12 @@ void feh_clean_exit(void)
 {
 	delete_rm_files();
 
+	free(opt.menu_bg);
+	free(opt.menu_font);
+
+	if(disp)
+		XCloseDisplay(disp);
+
 	if (opt.filelistfile)
 		feh_write_filelist(filelist, opt.filelistfile);
 
