@@ -396,6 +396,7 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		{"no-jump-on-resort", 0, 0, 220},
 #ifdef HAVE_LIBEXIF
 		{"draw-exif"     , 0, 0, 223},
+		{"auto-rotate"   , 0, 0, 242},
 #endif
 		{"cycle-once"    , 0, 0, 224},
 		{"no-xinerama"   , 0, 0, 225},
@@ -406,7 +407,8 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		{"keep-zoom-vp"  , 0, 0, 237},
 		{"scroll-step"   , 1, 0, 238},
 		{"xinerama-index", 1, 0, 239},
-
+		{"insecure"      , 0, 0, 240},
+		{"no-recursive"  , 0, 0, 241},
 		{0, 0, 0, 0}
 	};
 	int optch = 0, cmdx = 0;
@@ -513,6 +515,8 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 				opt.sort = SORT_NAME;
 			else if (!strcasecmp(optarg, "filename"))
 				opt.sort = SORT_FILENAME;
+			else if (!strcasecmp(optarg, "dirname"))
+				opt.sort = SORT_DIRNAME;
 			else if (!strcasecmp(optarg, "mtime"))
 				opt.sort = SORT_MTIME;
 			else if (!strcasecmp(optarg, "width"))
@@ -727,6 +731,9 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		case 223:
 			opt.draw_exif = 1;
 			break;
+		case 242:
+			opt.auto_rotate = 1;
+			break;
 #endif
 		case 224:
 			opt.cycle_once = 1;
@@ -759,6 +766,10 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		case 239:
 			opt.xinerama_index = atoi(optarg);
 			break;
+		case 240:
+			opt.insecure_ssl = 1;
+		case 241:
+			opt.recursive = 0;
 		default:
 			break;
 		}
@@ -785,10 +796,21 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 static void check_options(void)
 {
 	int i;
+	char *endptr;
+
 	for (i = 0; i < 10; i++) {
 		if (opt.actions[i] && !opt.hold_actions[i] && (opt.actions[i][0] == ';')) {
 			opt.hold_actions[i] = 1;
-			opt.actions[i] = &opt.actions[i][1];
+			opt.actions[i] = opt.actions[i] + 1;
+		}
+		opt.action_titles[i] = opt.actions[i];
+		if (opt.actions[i] && (opt.actions[i][0] == '[')) {
+			if (((endptr = strchr(opt.actions[i], ']')) != NULL)
+					&& (opt.actions[i][1] != ' ')) {
+				opt.action_titles[i] = opt.actions[i] + 1;
+				opt.actions[i] = endptr + 1;
+				*endptr = 0;
+			}
 		}
 	}
 
