@@ -51,6 +51,7 @@ struct __fehoptions {
 	unsigned char draw_filename;
 #ifdef HAVE_LIBEXIF
 	unsigned char draw_exif;
+	unsigned char auto_rotate;
 #endif
 	unsigned char list;
 	unsigned char quiet;
@@ -129,8 +130,10 @@ struct __fehoptions {
 };
 
 struct __fehkey {
-	int keysyms[3];
-	int keystates[3];
+	unsigned int keysyms[3];
+	unsigned int keystates[3];
+	unsigned int state;
+	unsigned int button;
 };
 
 struct __fehkb {
@@ -173,6 +176,7 @@ struct __fehkb {
 	struct __fehkey zoom_out;
 	struct __fehkey zoom_default;
 	struct __fehkey zoom_fit;
+	struct __fehkey zoom_fill;
 	struct __fehkey render;
 	struct __fehkey toggle_actions;
 	struct __fehkey toggle_filenames;
@@ -199,24 +203,11 @@ struct __fehkb {
 	struct __fehkey reload_minus;
 	struct __fehkey reload_plus;
 	struct __fehkey toggle_keep_vp;
-};
-
-struct __fehbutton {
-	int modifier;
-	char button;
-};
-
-struct __fehbb {
-	struct __fehbutton pan;
-	struct __fehbutton zoom;
-	struct __fehbutton reload;
-	struct __fehbutton prev;
-	struct __fehbutton next;
-	struct __fehbutton menu;
-	struct __fehbutton blur;
-	struct __fehbutton rotate;
-	struct __fehbutton zoom_in;
-	struct __fehbutton zoom_out;
+	struct __fehkey pan;
+	struct __fehkey zoom;
+	struct __fehkey reload;
+	struct __fehkey blur;
+	struct __fehkey rotate;
 };
 
 void init_parse_options(int argc, char **argv);
